﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Xml.Serialization;

namespace SerializationTest.Model
{
    /// <summary>
    /// 
    /// </summary>
    [XmlRoot("book")]
    public class BookInfo
    {
        /// <summary>
        /// 
        /// </summary>
        public BookInfo()
        {
            Name = "三体";
            Description = "刘慈欣系统科幻小说";
            prices.Add("CNY", 0.0M);
            Author = new PersonInfo() { Name = "刘慈欣" };
            Contributors.Add(new PersonInfo() { Name = "他老婆" });
            Contributors.Add(new PersonInfo() { Name = "他丈夫" });
        }
        
        private string name;

        [XmlAttribute("name")]
        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        private Dictionary<string, decimal> prices = new Dictionary<string, decimal>();

        /// <summary>
        /// 货币单位与金额定义的价格清单
        /// </summary>
        // XmlSerializer不支持Dictionary的序列化
        [XmlIgnore]
        public Dictionary<string, decimal> Prices
        {
            get { return prices; }
            set { prices = value; }
        }

        // XmlSerializer 自动无视没有set的属性。
        public bool IsFree
        {
            get { return prices.Any(p => p.Value == 0.0M); }
            // set { throw new NotSupportedException(); }
        }

        private string description;

        [XmlElement("summary")]
        public string Description
        {
            get { return description; }
            set { description = value; }
        }

        private PersonInfo author;

        [XmlElement("author")]
        public PersonInfo Author
        {
            get { return author; }
            set { author = value; }
        }

        private List<PersonInfo> contributors = new List<PersonInfo>();

        [XmlArray("contributors")]
        [XmlArrayItem("contributor")]
        public List<PersonInfo> Contributors
        {
            get { return contributors; }
            set { contributors = value; }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="info"></param>
        /// <param name="context"></param>
        public void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            throw new NotImplementedException();
        }
    }

    /// <summary>
    /// 父类的XmlRoot
    /// </summary>
    [Serializable]
    [XmlRoot("book")]
    public class SerializableBookInfo : BookInfo
    {
    }
}
