﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Runtime.Serialization;
using System.Xml;
using System.Xml.Schema;
using System.Xml.Serialization;

namespace SerializationTest.Model
{
    [DataContract]
    public enum Gender
    {
        [XmlEnum(Name = "man")]
        // 使用EnumMember的同时，要使用DataContract
        [EnumMember(Value = "man")]
        Man,
        [XmlEnum(Name = "woman")]
        [EnumMember(Value = "woman")]
        Woman,
    }

    public class ClassWithEnumValue
    {
        public Gender Gender { get; set; }
    }

    public class PublicClassWithoutAttribute
    {
        public string Name { get; set; }

        public string Value;

        internal int Age;

        private DateTime BirthDay;
    }

    [Serializable]
    public class PublicClassWithSerializableAttribute
    {
        public string Name { get; set; }

        public string Value;

        internal int Age;

        private DateTime BirthDay;
    }

    [DataContract]
    public class PublicClassWithAllContractAttribute
    {
        [DataMember]
        public string Name { get; set; }

        [DataMember]
        public string Value;

        [DataMember]
        internal int Age;

        [DataMember]
        private DateTime BirthDay;
    }

    [XmlRoot("class")]
    [DataContract(Name = "class", Namespace = "")]
    public class ClassSpecifySequence
    {
        [XmlElement(Order = 0, ElementName = "name")]
        [DataMember(Order = 0, Name = "name")]
        public string Name { get; set; }

        [XmlElement(Order = 1, ElementName = "value")]
        [DataMember(Order = 1, Name = "value")]
        public string Value { get; set; }
    }

    [XmlRoot("class")]
    [DataContract(Name = "class", Namespace = "")]
    public class ClassNotSpecifySequence
    {
        [XmlElement(ElementName = "name")]
        [DataMember(Name = "name")]
        public string Name { get; set; }

        [XmlElement(ElementName = "value")]
        [DataMember(Name = "value")]
        public string Value { get; set; }
    }

    [DataContract]
    public class ClassWithInternalPropertyToSerialize
    {
        [XmlElement]
        [DataMember]
        private int Age { get; set; }
    }

    [DataContract]
    public class ClassWithReadOnlyPropertyToSerialize
    {
        [DataMember]
        public string Name { get; private set; }

        public int Age { get; private set; }
    }

    [DataContract(Name = "class")]
    public class ContractClassWithCustomXmlName
    {
        public ContractClassWithCustomXmlName()
        {
            Name = String.Empty;
        }

        [DataMember(Name = "name")]
        public string Name { get; set; }
    }

    public class ClassWithXmlAttribute
    {
        public ClassWithXmlAttribute()
        {
            Name = String.Empty;
        }

        [XmlAttribute("Name")]
        public string Name { get; set; }
    }

    internal class InternalClassWithoutContractAttribute
    {
        public string Name { get; set; }

        public string Value;
    }

    [DataContract]
    public class PublicClassWithContractAttribute
    {
        public string Name { get; set; }

        public string Value;
    }

    public class ClassWithDictionary
    {
        public ClassWithDictionary()
        {
            people = new Dictionary<string, PersonInfo>()
            {
                { "1", new PersonInfo() { Name = "刘慈欣" } },
                { "2", new PersonInfo() { Name = "刘慈欣老婆" } },
            };
        }

        private Dictionary<string, PersonInfo> people;

        public Dictionary<string, PersonInfo> People
        {
            get { return people; }
            set { people = value; }
        }
    }

    public class ClassImplementsISerializable : ISerializable
    {
        public string Name { get; set; }

        public void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            throw new NotImplementedException();
        }
    }

    public class ClassImplementsIXmlSerializable : IXmlSerializable
    {
        /// <summary>
        /// 用于测试XmlElement和IXmlSerializable接口的优先级。
        /// </summary>
        [XmlElement]
        public string Name { get; set; }

        public XmlSchema GetSchema()
        {
            throw new NotImplementedException();
        }

        public void ReadXml(XmlReader reader)
        {
            if (reader.MoveToAttribute("name"))
            {
                if (reader.ReadAttributeValue())
                {
                    Name = reader.Value;
                }
            }
        }

        public void WriteXml(XmlWriter writer)
        {
            writer.WriteAttributeString("name", Name);
        }
    }


    [DataContract(IsReference = true)]
    public class ClassWithSelfCircle
    {
        public ClassWithSelfCircle()
        {
            if (Parent == null)
            {
                Child = new ClassWithSelfCircle(this);
            }
        }

        public ClassWithSelfCircle(ClassWithSelfCircle parent)
        {
            Parent = parent;
        }

        [DataMember]
        public ClassWithSelfCircle Child { get; set; }

        [DataMember]
        public ClassWithSelfCircle Parent { get; set; }
    }

    public class ClassWithCallBacks
    {
        public string Status { get; set; }

        [OnSerializing]
        internal void OnSerializingMethod(StreamingContext context)
        {
            Trace.TraceInformation("Serializing");
            Status = "Serializing";
        }

        [OnSerialized]
        internal void OnSerializedMethod(StreamingContext context)
        {
            Trace.TraceInformation("Serialized");
            Status = "Serialized";
        }

        [OnDeserialized]
        internal void OnDeserializedMethod(StreamingContext context)
        {
            Trace.TraceInformation("Deserialized");
        }

        [OnDeserializing]
        internal void OnDeserializingMethod(StreamingContext context)
        {
            Trace.TraceInformation("Deserializing");
        }
    }
}
