﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Runtime.Serialization;
using System.Xml;
using System.Xml.Schema;
using System.Xml.Serialization;

namespace SerializationTest.Model
{
    [Serializable]
    public class Staff : IDeserializationCallback
    {
        public string Name { get; set; }

        public int Age;

        internal decimal Salary;

        public bool RicherThan(Staff author)
        {
            return Salary > author.Salary;
        }

        public void OnDeserialization(object sender)
        {
            if (Name == null)
            {
                Name = "Deserialized";
            }
        }
    }

    [Serializable]
    [DataContract(IsReference = true)]
    public class LinkNode
    {
        public LinkNode()
        {
            Child = new List<LinkNode>();
        }

        public LinkNode(LinkNode child)
            : this()
        {
            child.Parent = this;
            Child.Add(child);
        }

        [DataMember]
        public ICollection<LinkNode> Child { get; set; }

        [DataMember]
        public LinkNode Parent { get; set; }
    }

    [Serializable]
    public class CustomISerialize : ISerializable
    {
        public void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            throw new NotImplementedException();
        }
    }

    [Serializable]
    public class SerializableClassWithCallbacks
    {
        public string Status { get; set; }

        [OnSerializing]
        internal void OnSerializingMethod(StreamingContext context)
        {
            Trace.TraceInformation("Serializing");
            Status = "Serializing";
        }

        [OnSerialized]
        internal void OnSerializedMethod(StreamingContext context)
        {
            Trace.TraceInformation("Serialized");
            Status = "Serialized";
        }

        [OnDeserialized]
        internal void OnDeserializedMethod(StreamingContext context)
        {
            Trace.TraceInformation("Deserialized");
        }

        [OnDeserializing]
        internal void OnDeserializingMethod(StreamingContext context)
        {
            Trace.TraceInformation("Deserializing");
        }
    }


    [Serializable]
    public class CustomXmlSerialize : IXmlSerializable
    {
        public string Name { get; set; }

        public string Value { get; set; }

        public string Address { get; set; }

        public XmlSchema GetSchema()
        {
            throw new NotImplementedException();
        }

        public void ReadXml(XmlReader reader)
        {
            reader.ReadStartElement();
            Name = reader.ReadElementContentAsString("Name", reader.NamespaceURI);
            Value = reader.ReadElementContentAsString("Value", reader.NamespaceURI);
            Address = reader.ReadElementContentAsString("Address", reader.NamespaceURI);
            reader.ReadEndElement();
        }

        public void WriteXml(XmlWriter writer)
        {
            writer.WriteElementString("Name", Name);
            writer.WriteElementString("Value", Value);
            writer.WriteElementString("Address", Address);
        }
    }

}
